#include <algorithm>
#include <map>
#include <regex>

#include "morse.h"

std::map<char, std::string> morseCode = {
    {'A', ".-"},     {'B', "-..."},    {'C', "-.-."},   {'D', "-.."},    {'E', "."},      {'F', "..-."},
    {'G', "--."},    {'H', "...."},    {'I', ".."},     {'J', ".---"},   {'K', "-.-"},    {'L', ".-.."},
    {'M', "--"},     {'N', "-."},      {'O', "---"},    {'P', ".--."},   {'Q', "--.-"},   {'R', ".-."},
    {'S', "..."},    {'T', "-"},       {'U', "..-"},    {'V', "...-"},   {'W', ".--"},    {'X', "-..-"},
    {'Y', "-.--"},   {'Z', "--.."},    {'1', ".----"},  {'2', "..---"},  {'3', "...--"},  {'4', "....-"},
    {'5', "....."},  {'6', "-...."},   {'7', "--..."},  {'8', "---.."},  {'9', "----."},  {'0', "-----"},
    {',', "--..--"}, {'.', ".-.-.-"},  {'?', "..--.."}, {'/', "-..-."},  {'-', "-....-"}, {':', "---..."},
    {'&', ".-..."},  {'\'', ".----."}, {'@', ".--.-."}, {')', "-.--.-"}, {'(', "-.--."},  {'\"', ".-..-."},
    {'=', "-...-"},  // '=' == <BT>
    {'k', "-.--."},  // k == <KN>
    {'s', "...-.-"}, // s == <SK>
    {'+', ".-.-."},  // + == <AR>
    {'a', "-.-.-"},  // a == <KA>
};

std::string refurbishMessage(const std::string &msg)
{
    std::string msgRefurb;
    msgRefurb.resize(msg.length());

    // Make the message all upper case
    std::transform(msg.cbegin(), msg.cend(), msgRefurb.begin(), [](unsigned char c) { return std::toupper(c); });

    // Encode the special characters as we like it
    msgRefurb = std::regex_replace(msgRefurb, std::regex("<BT>"), "=");
    msgRefurb = std::regex_replace(msgRefurb, std::regex("<KN>"), "k");
    msgRefurb = std::regex_replace(msgRefurb, std::regex("<SK>"), "s");
    msgRefurb = std::regex_replace(msgRefurb, std::regex("<AR>"), "+");
    msgRefurb = std::regex_replace(msgRefurb, std::regex("<KA>"), "a");

    // Remove all other unknown characters
    msgRefurb.erase(remove_if(msgRefurb.begin(), msgRefurb.end(),
                             [](const char &c) { return c != ' ' && morseCode.find(c) == morseCode.end(); }),
                   msgRefurb.end());

    // Remove spaces, if there are too many of them
    msgRefurb = std::regex_replace(msgRefurb, std::regex("(\\s+)"), " ");

    return msgRefurb;
}